@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,6 +32,7 @@
 
 #include <config.h>
 
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -47,6 +47,7 @@ static int delay_read_ms = 0;   /* read delay (milliseconds) */
 static int delay_write_ms = 0;  /* write delay (milliseconds) */
 static int delay_zero_ms = 0;   /* zero delay (milliseconds) */
 static int delay_trim_ms = 0;   /* trim delay (milliseconds) */
+static int delay_extents_ms = 0;/* extents delay (milliseconds) */
 
 static int
 parse_delay (const char *key, const char *value)
@@ -55,7 +56,7 @@ parse_delay (const char *key, const char *value)
   int r;
 
   if (len > 2 && strcmp (&value[len-2], "ms") == 0) {
-    if (sscanf (value, "%d", &r) == 1)
+    if (sscanf (value, "%d", &r) == 1 && r >= 0)
       return r;
     else {
       nbdkit_error ("cannot parse %s in milliseconds parameter: %s",
@@ -64,8 +65,14 @@ parse_delay (const char *key, const char *value)
     }
   }
   else {
-    if (sscanf (value, "%d", &r) == 1)
+    if (sscanf (value, "%d", &r) == 1 && r >= 0) {
+      if (r * 1000LL > INT_MAX) {
+        nbdkit_error ("seconds parameter %s is too large: %s",
+                      key, value);
+        return -1;
+      }
       return r * 1000;
+    }
     else {
       nbdkit_error ("cannot parse %s in seconds parameter: %s",
                     key, value);
@@ -74,40 +81,51 @@ parse_delay (const char *key, const char *value)
   }
 }
 
-static void
-delay (int ms)
+static int
+delay (int ms, int *err)
 {
   if (ms > 0) {
     const struct timespec ts = {
       .tv_sec = ms / 1000,
-      .tv_nsec = (ms * 1000000) % 1000000000
+      .tv_nsec = (ms % 1000) * 1000000,
     };
-    nanosleep (&ts, NULL);
+    if (nanosleep (&ts, NULL) == -1) {
+      nbdkit_error ("nanosleep: %m");
+      *err = errno;
+      return -1;
+    }
   }
+  return 0;
 }
 
-static void
-read_delay (void)
+static int
+read_delay (int *err)
 {
-  delay (delay_read_ms);
+  return delay (delay_read_ms, err);
 }
 
-static void
-write_delay (void)
+static int
+write_delay (int *err)
 {
-  delay (delay_write_ms);
+  return delay (delay_write_ms, err);
 }
 
-static void
-zero_delay (void)
+static int
+zero_delay (int *err)
 {
-  delay (delay_zero_ms);
+  return delay (delay_zero_ms, err);
 }
 
-static void
-trim_delay (void)
+static int
+trim_delay (int *err)
 {
-  delay (delay_trim_ms);
+  return delay (delay_trim_ms, err);
+}
+
+static int
+extents_delay (int *err)
+{
+  return delay (delay_extents_ms, err);
 }
 
 /* Called for each key=value passed on the command line. */
@@ -154,6 +172,13 @@ delay_config (nbdkit_next_config *next, void *nxdata,
       return -1;
     return 0;
   }
+  else if (strcmp (key, "delay-extent") == 0 ||
+           strcmp (key, "delay-extents") == 0) {
+    delay_extents_ms = parse_delay (key, value);
+    if (delay_extents_ms == -1)
+      return -1;
+    return 0;
+  }
   else
     return next (nxdata, key, value);
 }
@@ -164,6 +189,7 @@ delay_config (nbdkit_next_config *next, void *nxdata,
   "delay-write=<NN>[ms]           Write delay in seconds/milliseconds.\n" \
   "delay-zero=<NN>[ms]            Zero delay in seconds/milliseconds.\n" \
   "delay-trim=<NN>[ms]            Trim delay in seconds/milliseconds.\n" \
+  "delay-extents=<NN>[ms]         Extents delay in seconds/milliseconds.\n" \
   "wdelay=<NN>[ms]                Write, zero and trim delay in secs/msecs."
 
 /* Read data. */
@@ -172,7 +198,8 @@ delay_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
              void *handle, void *buf, uint32_t count, uint64_t offset,
              uint32_t flags, int *err)
 {
-  read_delay ();
+  if (read_delay (err) == -1)
+    return -1;
   return next_ops->pread (nxdata, buf, count, offset, flags, err);
 }
 
@@ -183,7 +210,8 @@ delay_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
               const void *buf, uint32_t count, uint64_t offset, uint32_t flags,
               int *err)
 {
-  write_delay ();
+  if (write_delay (err) == -1)
+    return -1;
   return next_ops->pwrite (nxdata, buf, count, offset, flags, err);
 }
 
@@ -193,7 +221,8 @@ delay_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
             void *handle, uint32_t count, uint64_t offset, uint32_t flags,
             int *err)
 {
-  zero_delay ();
+  if (zero_delay (err) == -1)
+    return -1;
   return next_ops->zero (nxdata, count, offset, flags, err);
 }
 
@@ -203,10 +232,22 @@ delay_trim (struct nbdkit_next_ops *next_ops, void *nxdata,
             void *handle, uint32_t count, uint64_t offset,
             uint32_t flags, int *err)
 {
-  trim_delay ();
+  if (trim_delay (err) == -1)
+    return -1;
   return next_ops->trim (nxdata, count, offset, flags, err);
 }
 
+/* Extents. */
+static int
+delay_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
+               void *handle, uint32_t count, uint64_t offset, uint32_t flags,
+               struct nbdkit_extents *extents, int *err)
+{
+  if (extents_delay (err) == -1)
+    return -1;
+  return next_ops->extents (nxdata, count, offset, flags, extents, err);
+}
+
 static struct nbdkit_filter filter = {
   .name              = "delay",
   .longname          = "nbdkit delay filter",
@@ -217,6 +258,7 @@ static struct nbdkit_filter filter = {
   .pwrite            = delay_pwrite,
   .zero              = delay_zero,
   .trim              = delay_trim,
+  .extents           = delay_extents,
 };
 
 NBDKIT_REGISTER_FILTER(filter)
