@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2014 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -44,6 +43,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <inttypes.h>
 #include <string.h>
@@ -58,9 +58,13 @@
 static const char *url = NULL;
 static const char *user = NULL;
 static char *password = NULL;
-static int sslverify = 1;
-static int timeout = 0;
+static const char *proxy_user = NULL;
+static char *proxy_password = NULL;
+static char *cookie = NULL;
+static bool sslverify = true;
+static long timeout = 0;
 static const char *unix_socket_path = NULL;
+static long protocols = CURLPROTO_ALL;
 
 /* Use '-D curl.verbose=1' to set. */
 int curl_debug_verbose = 0;
@@ -68,20 +72,99 @@ int curl_debug_verbose = 0;
 static void
 curl_load (void)
 {
-  curl_global_init (CURL_GLOBAL_DEFAULT);
+  CURLcode r;
+
+  r = curl_global_init (CURL_GLOBAL_DEFAULT);
+  if (r != CURLE_OK) {
+    nbdkit_error ("libcurl initialization failed: %d", (int) r);
+    exit (EXIT_FAILURE);
+  }
 }
 
 static void
 curl_unload (void)
 {
   free (password);
+  free (proxy_password);
+  free (cookie);
   curl_global_cleanup ();
 }
 
+/* See <curl/curl.h> */
+static struct { const char *name; long bitmask; } curl_protocols[] = {
+  { "http", CURLPROTO_HTTP },
+  { "https", CURLPROTO_HTTPS },
+  { "ftp", CURLPROTO_FTP },
+  { "ftps", CURLPROTO_FTPS },
+  { "scp", CURLPROTO_SCP },
+  { "sftp", CURLPROTO_SFTP },
+  { "telnet", CURLPROTO_TELNET },
+  { "ldap", CURLPROTO_LDAP },
+  { "ldaps", CURLPROTO_LDAPS },
+  { "dict", CURLPROTO_DICT },
+  { "file", CURLPROTO_FILE },
+  { "tftp", CURLPROTO_TFTP },
+  { "imap", CURLPROTO_IMAP },
+  { "imaps", CURLPROTO_IMAPS },
+  { "pop3", CURLPROTO_POP3 },
+  { "pop3s", CURLPROTO_POP3S },
+  { "smtp", CURLPROTO_SMTP },
+  { "smtps", CURLPROTO_SMTPS },
+  { "rtsp", CURLPROTO_RTSP },
+  { "rtmp", CURLPROTO_RTMP },
+  { "rtmpt", CURLPROTO_RTMPT },
+  { "rtmpe", CURLPROTO_RTMPE },
+  { "rtmpte", CURLPROTO_RTMPTE },
+  { "rtmps", CURLPROTO_RTMPS },
+  { "rtmpts", CURLPROTO_RTMPTS },
+  { "gopher", CURLPROTO_GOPHER },
+  { "smb", CURLPROTO_SMB },
+  { "smbs", CURLPROTO_SMBS },
+  { NULL }
+};
+
+/* Parse the protocols parameter. */
+static int
+parse_protocols (const char *value)
+{
+  size_t n, i;
+
+  protocols = 0;
+
+  while (*value) {
+    n = strcspn (value, ",");
+    for (i = 0; curl_protocols[i].name != NULL; ++i) {
+      if (strlen (curl_protocols[i].name) == n &&
+          strncmp (value, curl_protocols[i].name, n) == 0) {
+        protocols |= curl_protocols[i].bitmask;
+        goto found;
+      }
+    }
+    nbdkit_error ("protocols: protocol name not found: %.*s", (int) n, value);
+    return -1;
+
+  found:
+    value += n;
+    if (*value == ',')
+      value++;
+  }
+
+  if (protocols == 0) {
+    nbdkit_error ("protocols: empty list of protocols is not allowed");
+    return -1;
+  }
+
+  nbdkit_debug ("curl: protocols: %ld", protocols);
+
+  return 0;
+}
+
 /* Called for each key=value passed on the command line. */
 static int
 curl_config (const char *key, const char *value)
 {
+  int r;
+
   if (strcmp (key, "url") == 0)
     url = value;
 
@@ -94,23 +177,44 @@ curl_config (const char *key, const char *value)
       return -1;
   }
 
-  else if (strcmp (key, "sslverify") == 0) {
-    if (sscanf (value, "%d", &sslverify) != 1) {
-      nbdkit_error ("'sslverify' must be 0 or 1");
+  else if (strcmp (key, "proxy-user") == 0)
+    proxy_user = value;
+
+  else if (strcmp (key, "proxy-password") == 0) {
+    free (proxy_password);
+    if (nbdkit_read_password (value, &proxy_password) == -1)
       return -1;
-    }
+  }
+
+  else if (strcmp (key, "cookie") == 0) {
+    free (cookie);
+    if (nbdkit_read_password (value, &cookie) == -1)
+      return -1;
+  }
+
+  else if (strcmp (key, "sslverify") == 0) {
+    r = nbdkit_parse_bool (value);
+    if (r == -1)
+      return -1;
+    sslverify = r;
   }
 
   else if (strcmp (key, "timeout") == 0) {
-    if (sscanf (value, "%d", &timeout) != 1 || timeout < 0) {
+    if (sscanf (value, "%ld", &timeout) != 1 || timeout < 0) {
       nbdkit_error ("'timeout' must be 0 or a positive timeout in seconds");
       return -1;
     }
   }
 
-  else if (strcmp (key, "unix_socket_path") == 0)
+  else if (strcmp (key, "unix-socket-path") == 0 ||
+           strcmp (key, "unix_socket_path") == 0)
     unix_socket_path = value;
 
+  else if (strcmp (key, "protocols") == 0) {
+    if (parse_protocols (value) == -1)
+      return -1;
+  }
+
   else {
     nbdkit_error ("unknown parameter '%s'", key);
     return -1;
@@ -133,17 +237,21 @@ curl_config_complete (void)
 }
 
 #define curl_config_help \
-  "timeout=<TIMEOUT>          Set the timeout for requests (seconds).\n" \
+  "cookie=<COOKIE>            Set HTTP/HTTPS cookies.\n" \
   "password=<PASSWORD>        The password for the user account.\n" \
-  "sslverify=0                Do not verify SSL certificate of remote host.\n" \
-  "unix_socket_path=<PATH>    Open Unix domain socket instead of TCP/IP.\n" \
+  "protocols=PROTO,PROTO,..   Limit protocols allowed.\n" \
+  "proxy-password=<PASSWORD>  The proxy password.\n" \
+  "proxy-user=<USER>          The proxy user.\n" \
+  "timeout=<TIMEOUT>          Set the timeout for requests (seconds).\n" \
+  "sslverify=false            Do not verify SSL certificate of remote host.\n" \
+  "unix-socket-path=<PATH>    Open Unix domain socket instead of TCP/IP.\n" \
   "url=<URL>       (required) The disk image URL to serve.\n" \
   "user=<USER>                The user to log in as."
 
 /* The per-connection handle. */
 struct curl_handle {
   CURL *c;
-  int accept_range;
+  bool accept_range;
   int64_t exportsize;
   char errbuf[CURL_ERROR_SIZE];
   char *write_buf;
@@ -214,27 +322,34 @@ curl_open (int readonly)
 
   nbdkit_debug ("set libcurl URL: %s", url);
 
-  /* Other possible settings, which could be specified on the command line:
-CURLOPT_PROXY
-  */
   curl_easy_setopt (h->c, CURLOPT_AUTOREFERER, 1);
   curl_easy_setopt (h->c, CURLOPT_FOLLOWLOCATION, 1);
   curl_easy_setopt (h->c, CURLOPT_FAILONERROR, 1);
+  if (protocols != CURLPROTO_ALL) {
+    curl_easy_setopt (h->c, CURLOPT_PROTOCOLS, protocols);
+    curl_easy_setopt (h->c, CURLOPT_REDIR_PROTOCOLS, protocols);
+  }
   if (timeout > 0)
     curl_easy_setopt (h->c, CURLOPT_TIMEOUT, timeout);
-  if (sslverify == 0) {
+  if (!sslverify) {
     curl_easy_setopt (h->c, CURLOPT_SSL_VERIFYPEER, 0L);
     curl_easy_setopt (h->c, CURLOPT_SSL_VERIFYHOST, 0L);
   }
   if (user)
     curl_easy_setopt (h->c, CURLOPT_USERNAME, user);
   if (password)
-    curl_easy_setopt (h->c, CURLOPT_USERPWD, password);
+    curl_easy_setopt (h->c, CURLOPT_PASSWORD, password);
+  if (proxy_user)
+    curl_easy_setopt (h->c, CURLOPT_PROXYUSERNAME, proxy_user);
+  if (proxy_password)
+    curl_easy_setopt (h->c, CURLOPT_PROXYPASSWORD, proxy_password);
+  if (cookie)
+    curl_easy_setopt (h->c, CURLOPT_COOKIE, cookie);
 
   /* Get the file size and also whether the remote HTTP server
    * supports byte ranges.
    */
-  h->accept_range = 0;
+  h->accept_range = false;
   curl_easy_setopt (h->c, CURLOPT_NOBODY, 1); /* No Body, not nobody! */
   curl_easy_setopt (h->c, CURLOPT_HEADERFUNCTION, header_cb);
   curl_easy_setopt (h->c, CURLOPT_HEADERDATA, h);
@@ -303,7 +418,7 @@ header_cb (void *ptr, size_t size, size_t nmemb, void *opaque)
 
   if (realsize >= strlen (accept_line) &&
       strncmp (line, accept_line, strlen (accept_line)) == 0)
-    h->accept_range = 1;
+    h->accept_range = true;
 
   /* Useful to print the server headers when debugging.  However we
    * must strip off trailing \r?\n from each line.
