@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,10 +38,11 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/wait.h>
 
 #include <nbdkit-plugin.h>
 
+#include "utils.h"
+
 /* List of directories parsed from the command line. */
 static char **dirs = NULL;
 static size_t nr_dirs = 0;
@@ -59,8 +59,6 @@ static const char *params = NULL;
 static int fd = -1;
 
 /* Construct the temporary ISO. */
-static void shell_quote (const char *str, FILE *fp);
-
 static int
 make_iso (void)
 {
@@ -97,7 +95,8 @@ make_iso (void)
     return -1;
   }
 
-  fprintf (fp, "%s -quiet", isoprog);
+  shell_quote (isoprog, fp);
+  fprintf (fp, " -quiet");
   if (params)
     fprintf (fp, " %s", params);
   for (i = 0; i < nr_dirs; ++i) {
@@ -116,58 +115,12 @@ make_iso (void)
   nbdkit_debug ("%s", command);
   r = system (command);
   free (command);
-
-  if (WIFEXITED (r) && WEXITSTATUS (r) != 0) {
-    nbdkit_error ("external %s command failed with exit code %d",
-                  isoprog, WEXITSTATUS (r));
+  if (exit_status_to_nbd_error (r, isoprog) == -1)
     return -1;
-  }
-  else if (WIFSIGNALED (r)) {
-    nbdkit_error ("external %s command was killed by signal %d",
-                  isoprog, WTERMSIG (r));
-    return -1;
-  }
-  else if (WIFSTOPPED (r)) {
-    nbdkit_error ("external %s command was stopped by signal %d",
-                  isoprog, WSTOPSIG (r));
-    return -1;
-  }
 
   return 0;
 }
 
-/* Print str to fp, shell quoting if necessary.  This comes from
- * libguestfs, but was written by me so I'm relicensing it to a BSD
- * license for nbdkit.
- */
-static void
-shell_quote (const char *str, FILE *fp)
-{
-  const char *safe_chars =
-    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_=,:/";
-  size_t i, len;
-
-  /* If the string consists only of safe characters, output it as-is. */
-  len = strlen (str);
-  if (len == strspn (str, safe_chars)) {
-    fputs (str, fp);
-    return;
-  }
-
-  /* Double-quote the string. */
-  fputc ('"', fp);
-  for (i = 0; i < len; ++i) {
-    switch (str[i]) {
-    case '$': case '`': case '\\': case '"':
-      fputc ('\\', fp);
-      /*FALLTHROUGH*/
-    default:
-      fputc (str[i], fp);
-    }
-  }
-  fputc ('"', fp);
-}
-
 static void
 iso_unload (void)
 {
