@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2017-2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -269,7 +268,7 @@ nbd_request_raw (struct handle *h, uint16_t flags, uint16_t type,
 
   pthread_mutex_lock (&h->write_lock);
   nbdkit_debug ("sending request type %d (%s), flags %#x, offset %#" PRIx64
-                ", count %#x, cookie %#" PRIx64, type, name_of_nbd_cmd(type),
+                ", count %#x, cookie %#" PRIx64, type, name_of_nbd_cmd (type),
                 flags, offset, count, cookie);
   r = write_full (h->fd, &req, sizeof req);
   if (buf && !r)
@@ -345,7 +344,7 @@ nbd_request (struct handle *h, uint16_t flags, uint16_t type, uint64_t offset,
 static int
 nbd_reply_raw (struct handle *h, int *fd)
 {
-  struct reply rep;
+  struct simple_reply rep;
   struct transaction *trans;
   void *buf;
   uint32_t count;
@@ -353,7 +352,7 @@ nbd_reply_raw (struct handle *h, int *fd)
   *fd = -1;
   if (read_full (h->fd, &rep, sizeof rep) < 0)
     return nbd_mark_dead (h);
-  if (be32toh (rep.magic) != NBD_REPLY_MAGIC)
+  if (be32toh (rep.magic) != NBD_SIMPLE_REPLY_MAGIC)
     return nbd_mark_dead (h);
   nbdkit_debug ("received reply for cookie %#" PRIx64 ", status %s",
                 rep.handle, name_of_nbd_error(be32toh (rep.error)));
@@ -386,6 +385,8 @@ nbd_reply_raw (struct handle *h, int *fd)
     return EINVAL;
   case NBD_ENOSPC:
     return ENOSPC;
+  case NBD_EOVERFLOW:
+    return EOVERFLOW;
   case NBD_ESHUTDOWN:
     return ESHUTDOWN;
   }
@@ -485,7 +486,7 @@ nbd_open (int readonly)
     nbdkit_error ("unable to read magic: %m");
     goto err;
   }
-  if (strncmp(old.nbdmagic, "NBDMAGIC", sizeof old.nbdmagic)) {
+  if (strncmp (old.nbdmagic, "NBDMAGIC", sizeof old.nbdmagic)) {
     nbdkit_error ("wrong magic, %s is not an NBD server", sockname);
     goto err;
   }
@@ -511,7 +512,8 @@ nbd_open (int readonly)
       nbdkit_error ("unable to read global flags: %m");
       goto err;
     }
-    cflags = htobe32(gflags & (NBD_FLAG_FIXED_NEWSTYLE | NBD_FLAG_NO_ZEROES));
+    gflags = be16toh (gflags);
+    cflags = htobe32 (gflags & (NBD_FLAG_FIXED_NEWSTYLE | NBD_FLAG_NO_ZEROES));
     if (write_full (h->fd, &cflags, sizeof cflags)) {
       nbdkit_error ("unable to return global flags: %m");
       goto err;
@@ -749,4 +751,4 @@ static struct nbdkit_plugin plugin = {
   .errno_is_preserved = 1,
 };
 
-NBDKIT_REGISTER_PLUGIN(plugin)
+NBDKIT_REGISTER_PLUGIN (plugin)
