@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -35,6 +34,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <ctype.h>
 #include <inttypes.h>
@@ -45,12 +45,17 @@
 #include "byte-swapping.h"
 
 #include "efi-crc32.h"
+#include "gpt.h"
 #include "regions.h"
 #include "virtual-disk.h"
 
-static void create_gpt_partition_header (const void *pt, int is_primary, unsigned char *out);
+static void create_gpt_partition_header (const void *pt, bool is_primary,
+                                         unsigned char *out);
 static void create_gpt_partition_table (unsigned char *out);
-static void create_gpt_partition_table_entry (const struct region *region, int bootable, char partition_type_guid[16], unsigned char *out);
+static void create_gpt_partition_table_entry (const struct region *region,
+                                              bool bootable,
+                                              char partition_type_guid[16],
+                                              unsigned char *out);
 static void create_gpt_protective_mbr (unsigned char *out);
 
 void
@@ -66,43 +71,28 @@ create_gpt_layout (void)
   create_gpt_partition_table (pt);
 
   /* Partition table header.  LBA 1 */
-  create_gpt_partition_header (pt, 1, &primary[SECTOR_SIZE]);
+  create_gpt_partition_header (pt, true, &primary[SECTOR_SIZE]);
 
   /* Backup partition table.  LBA -(LBAs+2) */
   pt = secondary;
   create_gpt_partition_table (pt);
 
   /* Backup partition table header.  LBA -1 */
-  create_gpt_partition_header (pt, 0, &secondary[GPT_PTA_LBAs*SECTOR_SIZE]);
+  create_gpt_partition_header (pt, false, &secondary[GPT_PTA_LBAs*SECTOR_SIZE]);
 }
 
 static void
-create_gpt_partition_header (const void *pt, int is_primary,
+create_gpt_partition_header (const void *pt, bool is_primary,
                              unsigned char *out)
 {
   uint64_t nr_lbas;
-  struct gpt_header {
-    char signature[8];
-    char revision[4];
-    uint32_t header_size;
-    uint32_t crc;
-    uint32_t reserved;
-    uint64_t current_lba;
-    uint64_t backup_lba;
-    uint64_t first_usable_lba;
-    uint64_t last_usable_lba;
-    char guid[16];
-    uint64_t partition_entries_lba;
-    uint32_t nr_partition_entries;
-    uint32_t size_partition_entry;
-    uint32_t crc_partitions;
-  } *header = (struct gpt_header *) out;
+  struct gpt_header *header = (struct gpt_header *) out;
 
   nr_lbas = virtual_size (&regions) / SECTOR_SIZE;
 
   memset (header, 0, sizeof *header);
-  memcpy (header->signature, "EFI PART", 8);
-  memcpy (header->revision, "\0\0\1\0", 4); /* revision 1.0 */
+  memcpy (header->signature, GPT_SIGNATURE, sizeof (header->signature));
+  memcpy (header->revision, GPT_REVISION, sizeof (header->revision));
   header->header_size = htole32 (sizeof *header);
   if (is_primary) {
     header->current_lba = htole64 (1);
@@ -148,19 +138,13 @@ create_gpt_partition_table (unsigned char *out)
 
 static void
 create_gpt_partition_table_entry (const struct region *region,
-                                  int bootable, char partition_type_guid[16],
+                                  bool bootable,
+                                  char partition_type_guid[16],
                                   unsigned char *out)
 {
   size_t i, len;
   const char *filename;
-  struct gpt_entry {
-    char partition_type_guid[16];
-    char unique_guid[16];
-    uint64_t first_lba;
-    uint64_t last_lba;
-    uint64_t attributes;
-    char name[72];              /* UTF-16LE */
-  } *entry = (struct gpt_entry *) out;
+  struct gpt_entry *entry = (struct gpt_entry *) out;
 
   assert (sizeof (struct gpt_entry) == GPT_PT_ENTRY_SIZE);
 
