@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,6 +39,7 @@
 
 #include "rounding.h"
 
+#include "gpt.h"
 #include "regions.h"
 
 #define SECTOR_SIZE UINT64_C(512)
@@ -50,18 +50,6 @@
  */
 #define MAX_MBR_DISK_SIZE (UINT32_MAX * SECTOR_SIZE - 5 * MAX_ALIGNMENT)
 
-/* GPT_MIN_PARTITIONS is the minimum number of partitions and is
- * defined by the UEFI standard (assuming 512 byte sector size).  If
- * we are requested to allocate more than GPT_MIN_PARTITIONS then we
- * increase the partition table in chunks of this size.  Note that
- * clients may not support > GPT_MIN_PARTITIONS.
- *
- * GPT_PT_ENTRY_SIZE is the minimum specified by the UEFI spec, but
- * increasing it is not useful.
- */
-#define GPT_MIN_PARTITIONS 128
-#define GPT_PT_ENTRY_SIZE 128
-
 /* For GPT, the number of entries in the partition table array (PTA),
  * and the number of LBAs which the PTA occupies.  The latter will be
  * 32 if the number of files is <= GPT_MIN_PARTITIONS, which is the
@@ -104,7 +92,7 @@ extern struct file *files;
 extern size_t nr_files;
 
 extern struct regions regions;
-extern unsigned char *primary, *secondary;
+extern unsigned char *primary, *secondary, **ebr;
 
 /* Main entry point called after files array has been populated. */
 extern int create_virtual_disk_layout (void);
@@ -115,16 +103,16 @@ extern int create_virtual_disk_layout (void);
 extern int parse_guid (const char *str, char *out)
   __attribute__((__nonnull__ (1, 2)));
 
-/* Internal functions for creating MBR and GPT layouts.  These are
- * published here because the GPT code calls into the MBR code, but
- * are not meant to be called from the main plugin.
+/* Internal function for creating a single MBR PTE.  The GPT code
+ * calls this for creating the protective MBR.
  */
-extern void create_mbr_partition_table (unsigned char *out)
-  __attribute__((__nonnull__ (1)));
 extern void create_mbr_partition_table_entry (const struct region *,
                                               bool bootable, int partition_id,
                                               unsigned char *)
   __attribute__((__nonnull__ (1, 4)));
+
+/* Create MBR or GPT layout. */
+extern void create_mbr_layout (void);
 extern void create_gpt_layout (void);
 
 #endif /* NBDKIT_VIRTUAL_DISK_H */
