@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2013-2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -92,8 +91,10 @@ callback_defined (const char *name, PyObject **obj_rtn)
   assert (module != NULL);
 
   obj = PyObject_GetAttrString (module, name);
-  if (!obj)
+  if (!obj) {
+    PyErr_Clear (); /* Clear the AttributeError from testing attr. */
     return 0;
+  }
   if (!PyCallable_Check (obj)) {
     nbdkit_debug ("object %s isn't callable", name);
     Py_DECREF (obj);
