@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2013-2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -35,21 +34,29 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <inttypes.h>
 #include <string.h>
 #include <unistd.h>
 #include <dlfcn.h>
 
+#define NBDKIT_API_VERSION 2
+
 #include <nbdkit-plugin.h>
 
 #include "isaligned.h"
+#include "minmax.h"
+#include "rounding.h"
 
 #include "vddk-structs.h"
 
 /* Enable extra disk info debugging with: -D vddk.diskinfo=1 */
 int vddk_debug_diskinfo;
 
+/* Enable debugging of extents code with: -D vddk.extents=1 */
+int vddk_debug_extents;
+
 /* The VDDK APIs that we call.  These globals are initialized when the
  * plugin is loaded (by vddk_load).
  */
@@ -57,6 +64,8 @@ static char *(*VixDiskLib_GetErrorText) (VixError err, const char *unused);
 static void (*VixDiskLib_FreeErrorText) (char *text);
 static VixError (*VixDiskLib_InitEx) (uint32_t major, uint32_t minor, VixDiskLibGenericLogFunc *log_function, VixDiskLibGenericLogFunc *warn_function, VixDiskLibGenericLogFunc *panic_function, const char *lib_dir, const char *config_file);
 static void (*VixDiskLib_Exit) (void);
+static VixDiskLibConnectParams *(*VixDiskLib_AllocateConnectParams) (void);
+static void (*VixDiskLib_FreeConnectParams) (VixDiskLibConnectParams *params);
 static VixError (*VixDiskLib_ConnectEx) (const VixDiskLibConnectParams *params, char read_only, const char *snapshot_ref, const char *transport_modes, VixDiskLibConnection *connection);
 static VixError (*VixDiskLib_Open) (const VixDiskLibConnection connection, const char *path, uint32_t flags, VixDiskLibHandle *handle);
 static const char *(*VixDiskLib_GetTransportMode) (VixDiskLibHandle handle);
@@ -66,6 +75,9 @@ static VixError (*VixDiskLib_GetInfo) (VixDiskLibHandle handle, VixDiskLibInfo *
 static void (*VixDiskLib_FreeInfo) (VixDiskLibInfo *info);
 static VixError (*VixDiskLib_Read) (VixDiskLibHandle handle, uint64_t start_sector, uint64_t nr_sectors, unsigned char *buf);
 static VixError (*VixDiskLib_Write) (VixDiskLibHandle handle, uint64_t start_sector, uint64_t nr_sectors, const unsigned char *buf);
+static VixError (*VixDiskLib_Flush) (VixDiskLibHandle handle);
+static VixError (*VixDiskLib_QueryAllocatedBlocks) (VixDiskLibHandle diskHandle, uint64_t start_sector, uint64_t nr_sectors, uint64_t chunk_size, VixDiskLibBlockList **block_list);
+static VixError (*VixDiskLib_FreeBlockList) (VixDiskLibBlockList *block_list);
 
 /* Parameters passed to InitEx. */
 #define VDDK_MAJOR 5
@@ -82,12 +94,14 @@ static int nfc_host_port = 0;              /* nfchostport */
 static char *password = NULL;              /* password */
 static int port = 0;                       /* port */
 static const char *server_name = NULL;     /* server */
+static bool single_link = false;           /* single-link */
 static const char *snapshot_moref = NULL;  /* snapshot */
 static const char *thumb_print = NULL;     /* thumbprint */
 static const char *transport_modes = NULL; /* transports */
+static bool unbuffered = false;            /* unbuffered */
 static const char *username = NULL;        /* user */
 static const char *vmx_spec = NULL;        /* vm */
-static int is_remote = 0;
+static bool is_remote = false;
 
 #define VDDK_ERROR(err, fs, ...)                                \
   do {                                                          \
@@ -147,32 +161,74 @@ error_function (const char *fs, va_list args)
 static void
 vddk_load (void)
 {
-  static const char soname[] = "libvixDiskLib.so.6";
+  static const char *sonames[] = {
+    /* Prefer the newest library in case multiple exist. */
+    "libvixDiskLib.so.6",
+    "libvixDiskLib.so.5",
+  };
+  size_t i;
+  char *orig_error = NULL;
 
-  /* Load the plugin and set the entry points. */
-  dl = dlopen (soname, RTLD_NOW);
+  /* Load the library. */
+  for (i = 0; i < sizeof sonames / sizeof sonames[0]; ++i) {
+    dl = dlopen (sonames[i], RTLD_NOW);
+    if (dl != NULL)
+      break;
+    if (i == 0) {
+      orig_error = dlerror ();
+      if (orig_error)
+        orig_error = strdup (orig_error);
+    }
+  }
   if (dl == NULL) {
     nbdkit_error ("%s\n\n"
                   "If '%s' is located on a non-standard path you may need to\n"
                   "set $LD_LIBRARY_PATH or edit /etc/ld.so.conf.\n\n"
                   "See the nbdkit-vddk-plugin(1) man page for details.",
-                  dlerror (), soname);
+                  orig_error ? : "(unknown error)", sonames[0]);
+    free (orig_error);
     exit (EXIT_FAILURE);
   }
+  free (orig_error);
 
-  VixDiskLib_GetErrorText = dlsym (dl, "VixDiskLib_GetErrorText");
-  VixDiskLib_FreeErrorText = dlsym (dl, "VixDiskLib_FreeErrorText");
-  VixDiskLib_InitEx = dlsym (dl, "VixDiskLib_InitEx");
-  VixDiskLib_Exit = dlsym (dl, "VixDiskLib_Exit");
-  VixDiskLib_ConnectEx = dlsym (dl, "VixDiskLib_ConnectEx");
-  VixDiskLib_Open = dlsym (dl, "VixDiskLib_Open");
-  VixDiskLib_GetTransportMode = dlsym (dl, "VixDiskLib_GetTransportMode");
-  VixDiskLib_Close = dlsym (dl, "VixDiskLib_Close");
-  VixDiskLib_Disconnect = dlsym (dl, "VixDiskLib_Disconnect");
-  VixDiskLib_GetInfo = dlsym (dl, "VixDiskLib_GetInfo");
-  VixDiskLib_FreeInfo = dlsym (dl, "VixDiskLib_FreeInfo");
-  VixDiskLib_Read = dlsym (dl, "VixDiskLib_Read");
-  VixDiskLib_Write = dlsym (dl, "VixDiskLib_Write");
+  /* Non-optional symbols.  I have checked that all these exist in at
+   * least VDDK 5.1.1 (2011) which is the earliest version of VDDK
+   * that we support.
+   */
+#define LOAD(sym)                                                 \
+  do {                                                            \
+    sym = dlsym (dl, #sym);                                       \
+    if (sym == NULL) {                                            \
+      nbdkit_error ("required VDDK symbol \"%s\" is missing: %s", \
+                    #sym, dlerror ());                            \
+      exit (EXIT_FAILURE);                                        \
+    }                                                             \
+  } while (0)
+  LOAD (VixDiskLib_GetErrorText);
+  LOAD (VixDiskLib_FreeErrorText);
+  LOAD (VixDiskLib_InitEx);
+  LOAD (VixDiskLib_Exit);
+  LOAD (VixDiskLib_ConnectEx);
+  LOAD (VixDiskLib_Open);
+  LOAD (VixDiskLib_GetTransportMode);
+  LOAD (VixDiskLib_Close);
+  LOAD (VixDiskLib_Disconnect);
+  LOAD (VixDiskLib_GetInfo);
+  LOAD (VixDiskLib_FreeInfo);
+  LOAD (VixDiskLib_Read);
+  LOAD (VixDiskLib_Write);
+  LOAD (VixDiskLib_FreeConnectParams);
+#undef LOAD
+
+#define LOAD_OPTIONAL(sym) sym = dlsym (dl, #sym)
+  /* Added in VDDK 6.0, this will be NULL in earlier versions. */
+  LOAD_OPTIONAL (VixDiskLib_Flush);
+
+  /* Added in VDDK 6.7, these will be NULL for earlier versions: */
+  LOAD_OPTIONAL (VixDiskLib_QueryAllocatedBlocks);
+  LOAD_OPTIONAL (VixDiskLib_FreeBlockList);
+  LOAD_OPTIONAL (VixDiskLib_AllocateConnectParams);
+#undef LOAD_OPTIONAL
 }
 
 static void
@@ -237,6 +293,13 @@ vddk_config (const char *key, const char *value)
   else if (strcmp (key, "server") == 0) {
     server_name = value;
   }
+  else if (strcmp (key, "single-link") == 0) {
+    int r = nbdkit_parse_bool (value);
+
+    if (r == -1)
+      return -1;
+    single_link = r;
+  }
   else if (strcmp (key, "snapshot") == 0) {
     snapshot_moref = value;
   }
@@ -246,6 +309,13 @@ vddk_config (const char *key, const char *value)
   else if (strcmp (key, "transports") == 0) {
     transport_modes = value;
   }
+  else if (strcmp (key, "unbuffered") == 0) {
+    int r = nbdkit_parse_bool (value);
+
+    if (r == -1)
+      return -1;
+    unbuffered = r;
+  }
   else if (strcmp (key, "user") == 0) {
     username = value;
   }
@@ -359,10 +429,40 @@ vddk_dump_plugin (void)
 
 /* The per-connection handle. */
 struct vddk_handle {
+  VixDiskLibConnectParams *params; /* connection parameters */
   VixDiskLibConnection connection; /* connection */
   VixDiskLibHandle handle;         /* disk handle */
 };
 
+static inline VixDiskLibConnectParams *
+allocate_connect_params (void)
+{
+  VixDiskLibConnectParams *ret;
+
+  if (VixDiskLib_AllocateConnectParams != NULL) {
+    DEBUG_CALL ("VixDiskLib_AllocateConnectParams", "");
+    ret = VixDiskLib_AllocateConnectParams ();
+  }
+  else
+    ret = calloc (1, sizeof (VixDiskLibConnectParams));
+
+  return ret;
+}
+
+static inline void
+free_connect_params (VixDiskLibConnectParams *params)
+{
+  /* Only use FreeConnectParams if AllocateConnectParams was
+   * originally called.  Otherwise use free.
+   */
+  if (VixDiskLib_AllocateConnectParams != NULL) {
+    DEBUG_CALL ("VixDiskLib_FreeConnectParams", "params");
+    VixDiskLib_FreeConnectParams (params);
+  }
+  else
+    free (params);
+}
+
 /* Create the per-connection handle. */
 static void *
 vddk_open (int readonly)
@@ -370,7 +470,6 @@ vddk_open (int readonly)
   struct vddk_handle *h;
   VixError err;
   uint32_t flags;
-  VixDiskLibConnectParams params;
 
   h = malloc (sizeof *h);
   if (h == NULL) {
@@ -378,24 +477,29 @@ vddk_open (int readonly)
     return NULL;
   }
 
-  memset (&params, 0, sizeof params);
+  h->params = allocate_connect_params ();
+  if (h->params == NULL) {
+    nbdkit_error ("allocate VixDiskLibConnectParams: %m");
+    goto err0;
+  }
+
   if (is_remote) {
-    params.vmxSpec = (char *) vmx_spec;
-    params.serverName = (char *) server_name;
+    h->params->vmxSpec = (char *) vmx_spec;
+    h->params->serverName = (char *) server_name;
     if (cookie == NULL) {
-      params.credType = VIXDISKLIB_CRED_UID;
-      params.creds.uid.userName = (char *) username;
-      params.creds.uid.password = password;
+      h->params->credType = VIXDISKLIB_CRED_UID;
+      h->params->creds.uid.userName = (char *) username;
+      h->params->creds.uid.password = password;
     }
     else {
-      params.credType = VIXDISKLIB_CRED_SESSIONID;
-      params.creds.sessionId.cookie = (char *) cookie;
-      params.creds.sessionId.userName = (char *) username;
-      params.creds.sessionId.key = password;
+      h->params->credType = VIXDISKLIB_CRED_SESSIONID;
+      h->params->creds.sessionId.cookie = (char *) cookie;
+      h->params->creds.sessionId.userName = (char *) username;
+      h->params->creds.sessionId.key = password;
     }
-    params.thumbPrint = (char *) thumb_print;
-    params.port = port;
-    params.nfcHostPort = nfc_host_port;
+    h->params->thumbPrint = (char *) thumb_print;
+    h->params->port = port;
+    h->params->nfcHostPort = nfc_host_port;
   }
 
   /* XXX Some documentation suggests we should call
@@ -405,11 +509,11 @@ vddk_open (int readonly)
    */
 
   DEBUG_CALL ("VixDiskLib_ConnectEx",
-              "&params, %d, %s, %s, &connection",
+              "h->params, %d, %s, %s, &connection",
               readonly,
               snapshot_moref ? : "NULL",
               transport_modes ? : "NULL");
-  err = VixDiskLib_ConnectEx (&params,
+  err = VixDiskLib_ConnectEx (h->params,
                               readonly,
                               snapshot_moref,
                               transport_modes,
@@ -422,6 +526,10 @@ vddk_open (int readonly)
   flags = 0;
   if (readonly)
     flags |= VIXDISKLIB_FLAG_OPEN_READ_ONLY;
+  if (single_link)
+    flags |= VIXDISKLIB_FLAG_OPEN_SINGLE_LINK;
+  if (unbuffered)
+    flags |= VIXDISKLIB_FLAG_OPEN_UNBUFFERED;
 
   DEBUG_CALL ("VixDiskLib_Open",
               "connection, %s, %d, &handle", filename, flags);
@@ -440,6 +548,8 @@ vddk_open (int readonly)
   DEBUG_CALL ("VixDiskLib_Disconnect", "connection");
   VixDiskLib_Disconnect (h->connection);
  err1:
+  free_connect_params (h->params);
+ err0:
   free (h);
   return NULL;
 }
@@ -450,6 +560,7 @@ vddk_close (void *handle)
 {
   struct vddk_handle *h = handle;
 
+  free_connect_params (h->params);
   DEBUG_CALL ("VixDiskLib_Close", "handle");
   VixDiskLib_Close (h->handle);
   DEBUG_CALL ("VixDiskLib_Disconnect", "connection");
@@ -506,7 +617,8 @@ vddk_get_size (void *handle)
  * Note that reads have to be aligned to sectors (XXX).
  */
 static int
-vddk_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
+vddk_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
+            uint32_t flags)
 {
   struct vddk_handle *h = handle;
   VixError err;
@@ -535,13 +647,17 @@ vddk_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
   return 0;
 }
 
+static int vddk_flush (void *handle, uint32_t flags);
+
 /* Write data to the file.
  *
  * Note that writes have to be aligned to sectors (XXX).
  */
 static int
-vddk_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset)
+vddk_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
+             uint32_t flags)
 {
+  const bool fua = flags & NBDKIT_FLAG_FUA;
   struct vddk_handle *h = handle;
   VixError err;
 
@@ -566,6 +682,193 @@ vddk_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset)
     return -1;
   }
 
+  if (fua && vddk_flush (handle, 0) == -1)
+    return -1;
+
+  return 0;
+}
+
+/* Flush data to the file. */
+static int
+vddk_flush (void *handle, uint32_t flags)
+{
+  struct vddk_handle *h = handle;
+  VixError err;
+
+  /* The Flush call was not available in VDDK < 6.0 so this is simply
+   * ignored on earlier versions.
+   */
+  if (VixDiskLib_Flush == NULL)
+    return 0;
+
+  DEBUG_CALL ("VixDiskLib_Flush", "handle");
+  err = VixDiskLib_Flush (h->handle);
+  if (err != VIX_OK) {
+    VDDK_ERROR (err, "VixDiskLib_Flush");
+    return -1;
+  }
+
+  return 0;
+}
+
+static int
+vddk_can_extents (void *handle)
+{
+  struct vddk_handle *h = handle;
+  VixError err;
+  VixDiskLibBlockList *block_list;
+
+  /* This call was added in VDDK 6.7.  In earlier versions the
+   * function pointer will be NULL and we cannot query extents.
+   */
+  if (VixDiskLib_QueryAllocatedBlocks == NULL) {
+    nbdkit_debug ("can_extents: VixDiskLib_QueryAllocatedBlocks == NULL, "
+                  "probably this is VDDK < 6.7");
+    return 0;
+  }
+
+  /* However even when the call is available it rarely works well so
+   * the best thing we can do here is to try the call and if it's
+   * non-functional return false.
+   */
+  DEBUG_CALL ("VixDiskLib_QueryAllocatedBlocks",
+              "handle, 0, %d sectors, %d sectors",
+              VIXDISKLIB_MIN_CHUNK_SIZE, VIXDISKLIB_MIN_CHUNK_SIZE);
+  err = VixDiskLib_QueryAllocatedBlocks (h->handle,
+                                         0, VIXDISKLIB_MIN_CHUNK_SIZE,
+                                         VIXDISKLIB_MIN_CHUNK_SIZE,
+                                         &block_list);
+  if (err == VIX_OK) {
+    DEBUG_CALL ("VixDiskLib_FreeBlockList", "block_list");
+    VixDiskLib_FreeBlockList (block_list);
+  }
+  if (err != VIX_OK) {
+    char *errmsg = VixDiskLib_GetErrorText (err, NULL);
+    nbdkit_debug ("can_extents: VixDiskLib_QueryAllocatedBlocks test failed, "
+                  "extents support will be disabled: "
+                  "original error: %s",
+                  errmsg);
+    VixDiskLib_FreeErrorText (errmsg);
+    return 0;
+  }
+
+  return 1;
+}
+
+static int
+add_extent (struct nbdkit_extents *extents,
+            uint64_t *position, uint64_t next_position, bool is_hole)
+{
+  uint32_t type = 0;
+  const uint64_t length = next_position - *position;
+
+  if (is_hole) {
+    type = NBDKIT_EXTENT_HOLE;
+    /* Images opened as single link might be backed by another file in the
+       chain, so the holes are not guaranteed to be zeroes. */
+    if (!single_link)
+      type |= NBDKIT_EXTENT_ZERO;
+  }
+
+  assert (*position <= next_position);
+  if (*position == next_position)
+    return 0;
+
+  if (vddk_debug_extents)
+    nbdkit_debug ("adding extent type %s at [%" PRIu64 "...%" PRIu64 "]",
+                  is_hole ? "hole" : "allocated data",
+                  *position, next_position-1);
+  if (nbdkit_add_extent (extents, *position, length, type) == -1)
+    return -1;
+
+  *position = next_position;
+  return 0;
+}
+
+static int
+vddk_extents (void *handle, uint32_t count, uint64_t offset, uint32_t flags,
+              struct nbdkit_extents *extents)
+{
+  struct vddk_handle *h = handle;
+  bool req_one = flags & NBDKIT_FLAG_REQ_ONE;
+  uint64_t position, end, start_sector;
+
+  position = offset;
+  end = offset + count;
+
+  /* We can only query whole chunks.  Therefore start with the first
+   * chunk before offset.
+   */
+  start_sector =
+    ROUND_DOWN (offset, VIXDISKLIB_MIN_CHUNK_SIZE * VIXDISKLIB_SECTOR_SIZE)
+    / VIXDISKLIB_SECTOR_SIZE;
+  while (start_sector * VIXDISKLIB_SECTOR_SIZE < end) {
+    VixError err;
+    uint32_t i;
+    uint64_t nr_chunks, nr_sectors;
+    VixDiskLibBlockList *block_list;
+
+    assert (IS_ALIGNED (start_sector, VIXDISKLIB_MIN_CHUNK_SIZE));
+
+    nr_chunks =
+      ROUND_UP (end - start_sector * VIXDISKLIB_SECTOR_SIZE,
+                VIXDISKLIB_MIN_CHUNK_SIZE * VIXDISKLIB_SECTOR_SIZE)
+      / (VIXDISKLIB_MIN_CHUNK_SIZE * VIXDISKLIB_SECTOR_SIZE);
+    nr_chunks = MIN (nr_chunks, VIXDISKLIB_MAX_CHUNK_NUMBER);
+    nr_sectors = nr_chunks * VIXDISKLIB_MIN_CHUNK_SIZE;
+
+    DEBUG_CALL ("VixDiskLib_QueryAllocatedBlocks",
+                "handle, %" PRIu64 " sectors, %" PRIu64 " sectors, "
+                "%d sectors",
+                start_sector, nr_sectors, VIXDISKLIB_MIN_CHUNK_SIZE);
+    err = VixDiskLib_QueryAllocatedBlocks (h->handle,
+                                           start_sector, nr_sectors,
+                                           VIXDISKLIB_MIN_CHUNK_SIZE,
+                                           &block_list);
+    if (err != VIX_OK) {
+      VDDK_ERROR (err, "VixDiskLib_QueryAllocatedBlocks");
+      return -1;
+    }
+
+    for (i = 0; i < block_list->numBlocks; ++i) {
+      uint64_t blk_offset, blk_length;
+
+      blk_offset = block_list->blocks[i].offset * VIXDISKLIB_SECTOR_SIZE;
+      blk_length = block_list->blocks[i].length * VIXDISKLIB_SECTOR_SIZE;
+
+      /* The query returns allocated blocks.  We must insert holes
+       * between the blocks as necessary.
+       */
+      if ((position < blk_offset &&
+           add_extent (extents, &position, blk_offset, true) == -1) ||
+          (add_extent (extents,
+                       &position, blk_offset + blk_length, false) == -1)) {
+        DEBUG_CALL ("VixDiskLib_FreeBlockList", "block_list");
+        VixDiskLib_FreeBlockList (block_list);
+        return -1;
+      }
+    }
+    DEBUG_CALL ("VixDiskLib_FreeBlockList", "block_list");
+    VixDiskLib_FreeBlockList (block_list);
+
+    /* There's an implicit hole after the returned list of blocks, up
+     * to the end of the QueryAllocatedBlocks request.
+     */
+    if (add_extent (extents,
+                    &position,
+                    (start_sector + nr_sectors) * VIXDISKLIB_SECTOR_SIZE,
+                    true) == -1)
+      return -1;
+
+    start_sector += nr_sectors;
+
+    /* If one extent was requested, as long as we've added an extent
+     * overlapping the original offset we're done.
+     */
+    if (req_one && position > offset)
+      break;
+  }
+
   return 0;
 }
 
@@ -584,6 +887,9 @@ static struct nbdkit_plugin plugin = {
   .get_size          = vddk_get_size,
   .pread             = vddk_pread,
   .pwrite            = vddk_pwrite,
+  .flush             = vddk_flush,
+  .can_extents       = vddk_can_extents,
+  .extents           = vddk_extents,
 };
 
 NBDKIT_REGISTER_PLUGIN(plugin)
