@@ -1,6 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2018 Red Hat Inc.
- * All rights reserved.
+ * Copyright (C) 2013-2019 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -139,6 +138,8 @@ extern void change_user (void);
 /* cleanup.c */
 extern void cleanup_free (void *ptr);
 #define CLEANUP_FREE __attribute__((cleanup (cleanup_free)))
+extern void cleanup_extents_free (void *ptr);
+#define CLEANUP_EXTENTS_FREE __attribute__((cleanup (cleanup_extents_free)))
 extern void cleanup_unlock (pthread_mutex_t **ptr);
 #define CLEANUP_UNLOCK __attribute__((cleanup (cleanup_unlock)))
 #define ACQUIRE_LOCK_FOR_CURRENT_SCOPE(mutex) \
@@ -147,6 +148,7 @@ extern void cleanup_unlock (pthread_mutex_t **ptr);
 
 /* connections.c */
 struct connection;
+
 typedef int (*connection_recv_function) (struct connection *,
                                          void *buf, size_t len)
   __attribute__((__nonnull__ (1, 2)));
@@ -155,29 +157,75 @@ typedef int (*connection_send_function) (struct connection *,
   __attribute__((__nonnull__ (1, 2)));
 typedef void (*connection_close_function) (struct connection *)
   __attribute__((__nonnull__ (1)));
+
+struct connection {
+  pthread_mutex_t request_lock;
+  pthread_mutex_t read_lock;
+  pthread_mutex_t write_lock;
+  pthread_mutex_t status_lock;
+  int status; /* 1 for more I/O with client, 0 for shutdown, -1 on error */
+  void *crypto_session;
+  int nworkers;
+
+  void **handles;
+  size_t nr_handles;
+
+  uint32_t cflags;
+  uint64_t exportsize;
+  uint16_t eflags;
+  bool readonly;
+  bool can_flush;
+  bool is_rotational;
+  bool can_trim;
+  bool can_zero;
+  bool can_fua;
+  bool can_multi_conn;
+  bool can_extents;
+  bool using_tls;
+  bool structured_replies;
+  bool meta_context_base_allocation;
+
+  int sockin, sockout;
+  connection_recv_function recv;
+  connection_send_function send;
+  connection_close_function close;
+};
+
 extern int handle_single_connection (int sockin, int sockout);
 extern int connection_set_handle (struct connection *conn,
                                   size_t i, void *handle)
   __attribute__((__nonnull__ (1 /* not 3 */)));
 extern void *connection_get_handle (struct connection *conn, size_t i)
   __attribute__((__nonnull__ (1)));
-extern pthread_mutex_t *connection_get_request_lock (struct connection *conn)
+extern int connection_get_status (struct connection *conn)
   __attribute__((__nonnull__ (1)));
-extern void connection_set_crypto_session (struct connection *conn,
-                                           void *session)
-  __attribute__((__nonnull__ (1 /* not 2 */)));
-extern void *connection_get_crypto_session (struct connection *conn)
+extern int connection_set_status (struct connection *conn, int value)
   __attribute__((__nonnull__ (1)));
-extern void connection_set_recv (struct connection *,
-                                 connection_recv_function)
-  __attribute__((__nonnull__ (1, 2)));
-extern void connection_set_send (struct connection *,
-                                 connection_send_function)
-  __attribute__((__nonnull__ (1, 2)));
-extern void connection_set_close (struct connection *,
-                                  connection_close_function)
+
+/* protocol-handshake.c */
+extern int protocol_handshake (struct connection *conn)
+  __attribute__((__nonnull__ (1)));
+extern int protocol_compute_eflags (struct connection *conn, uint16_t *flags)
   __attribute__((__nonnull__ (1, 2)));
 
+/* protocol-handshake-oldstyle.c */
+extern int protocol_handshake_oldstyle (struct connection *conn)
+  __attribute__((__nonnull__ (1)));
+
+/* protocol-handshake-newstyle.c */
+extern int protocol_handshake_newstyle (struct connection *conn)
+  __attribute__((__nonnull__ (1)));
+
+/* protocol.c */
+extern int protocol_recv_request_send_reply (struct connection *conn)
+  __attribute__((__nonnull__ (1)));
+
+/* The context ID of base:allocation.  As far as I can tell it doesn't
+ * matter what this is as long as nbdkit always returns the same
+ * number.
+ */
+#define base_allocation_id 1
+
 /* crypto.c */
 #define root_tls_certificates_dir sysconfdir "/pki/" PACKAGE_NAME
 extern void crypto_init (bool tls_set_on_cli);
@@ -235,6 +283,7 @@ struct backend {
   int (*is_rotational) (struct backend *, struct connection *conn);
   int (*can_trim) (struct backend *, struct connection *conn);
   int (*can_zero) (struct backend *, struct connection *conn);
+  int (*can_extents) (struct backend *, struct connection *conn);
   int (*can_fua) (struct backend *, struct connection *conn);
   int (*can_multi_conn) (struct backend *, struct connection *conn);
 
@@ -248,6 +297,9 @@ struct backend {
                uint64_t offset, uint32_t flags, int *err);
   int (*zero) (struct backend *, struct connection *conn, uint32_t count,
                uint64_t offset, uint32_t flags, int *err);
+  int (*extents) (struct backend *, struct connection *conn, uint32_t count,
+                  uint64_t offset, uint32_t flags,
+                  struct nbdkit_extents *extents, int *err);
 };
 
 /* plugins.c */
@@ -289,15 +341,16 @@ extern void threadlocal_init (void);
 extern void threadlocal_new_server_thread (void);
 extern void threadlocal_set_name (const char *name)
   __attribute__((__nonnull__ (1)));
+extern const char *threadlocal_get_name (void);
 extern void threadlocal_set_instance_num (size_t instance_num);
+extern size_t threadlocal_get_instance_num (void);
 extern void threadlocal_set_sockaddr (const struct sockaddr *addr,
                                       socklen_t addrlen)
   __attribute__((__nonnull__ (1)));
-extern const char *threadlocal_get_name (void);
-extern size_t threadlocal_get_instance_num (void);
+/*extern void threadlocal_get_sockaddr ();*/
 extern void threadlocal_set_error (int err);
 extern int threadlocal_get_error (void);
-/*extern void threadlocal_get_sockaddr ();*/
+extern void *threadlocal_buffer (size_t size);
 
 /* Declare program_name. */
 #if HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME == 1
