@@ -1,6 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2018 Red Hat Inc.
- * All rights reserved.
+ * Copyright (C) 2013-2019 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -44,6 +43,7 @@
 #include <dlfcn.h>
 
 #include "internal.h"
+#include "minmax.h"
 
 /* Maximum read or write request that we will handle. */
 #define MAX_REQUEST_SIZE (64 * 1024 * 1024)
@@ -191,6 +191,7 @@ plugin_dump_fields (struct backend *b)
   HAS (_flush_old);
   HAS (_trim_old);
   HAS (_zero_old);
+  HAS (can_zero);
   HAS (can_fua);
   HAS (pread);
   HAS (pwrite);
@@ -198,6 +199,8 @@ plugin_dump_fields (struct backend *b)
   HAS (trim);
   HAS (zero);
   HAS (can_multi_conn);
+  HAS (can_extents);
+  HAS (extents);
 #undef HAS
 
   /* Custom fields. */
@@ -391,6 +394,21 @@ plugin_can_zero (struct backend *b, struct connection *conn)
   return plugin_can_write (b, conn);
 }
 
+static int
+plugin_can_extents (struct backend *b, struct connection *conn)
+{
+  struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
+
+  assert (connection_get_handle (conn, 0));
+
+  debug ("can_extents");
+
+  if (p->plugin.can_extents)
+    return p->plugin.can_extents (connection_get_handle (conn, 0));
+  else
+    return p->plugin.extents != NULL;
+}
+
 static int
 plugin_can_fua (struct backend *b, struct connection *conn)
 {
@@ -577,8 +595,6 @@ plugin_zero (struct backend *b, struct connection *conn,
              uint32_t count, uint64_t offset, uint32_t flags, int *err)
 {
   struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
-  char *buf;
-  uint32_t limit;
   int r = -1;
   bool may_trim = flags & NBDKIT_FLAG_MAY_TRIM;
   bool fua = flags & NBDKIT_FLAG_FUA;
@@ -622,24 +638,21 @@ plugin_zero (struct backend *b, struct connection *conn,
   assert (p->plugin.pwrite || p->plugin._pwrite_old);
   flags &= ~NBDKIT_FLAG_MAY_TRIM;
   threadlocal_set_error (0);
-  limit = count < MAX_REQUEST_SIZE ? count : MAX_REQUEST_SIZE;
-  buf = calloc (limit, 1);
-  if (!buf) {
-    *err = ENOMEM;
-    return -1;
-  }
 
   while (count) {
+    /* Always contains zeroes, but we can't use const or else gcc 9
+     * will use .rodata instead of .bss and inflate the binary size.
+     */
+    static /* const */ char buf[MAX_REQUEST_SIZE];
+    uint32_t limit = MIN (count, sizeof buf);
+
     r = plugin_pwrite (b, conn, buf, limit, offset, flags, err);
     if (r == -1)
       break;
     count -= limit;
-    if (count < limit)
-      limit = count;
   }
 
   *err = errno;
-  free (buf);
   errno = *err;
 
  done:
@@ -650,6 +663,39 @@ plugin_zero (struct backend *b, struct connection *conn,
   return r;
 }
 
+static int
+plugin_extents (struct backend *b, struct connection *conn,
+                uint32_t count, uint64_t offset, uint32_t flags,
+                struct nbdkit_extents *extents, int *err)
+{
+  struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
+  bool req_one = flags & NBDKIT_FLAG_REQ_ONE;
+  int r;
+
+  assert (connection_get_handle (conn, 0));
+  assert (!(flags & ~NBDKIT_FLAG_REQ_ONE));
+
+  /* This should be true because plugin_can_extents checks it. */
+  assert (p->plugin.extents);
+
+  debug ("extents count=%" PRIu32 " offset=%" PRIu64 " req_one=%d",
+         count, offset, req_one);
+
+  if (!count)
+    return 0;
+
+  r = p->plugin.extents (connection_get_handle (conn, 0), count, offset,
+                         flags, extents);
+  if (r >= 0 && nbdkit_extents_count (extents) < 1) {
+    nbdkit_error ("extents: plugin must return at least one extent");
+    nbdkit_set_error (EINVAL);
+    r = -1;
+  }
+  if (r == -1)
+    *err = get_error (p);
+  return r;
+}
+
 static struct backend plugin_functions = {
   .free = plugin_free,
   .thread_model = plugin_thread_model,
@@ -671,6 +717,7 @@ static struct backend plugin_functions = {
   .is_rotational = plugin_is_rotational,
   .can_trim = plugin_can_trim,
   .can_zero = plugin_can_zero,
+  .can_extents = plugin_can_extents,
   .can_fua = plugin_can_fua,
   .can_multi_conn = plugin_can_multi_conn,
   .pread = plugin_pread,
@@ -678,6 +725,7 @@ static struct backend plugin_functions = {
   .flush = plugin_flush,
   .trim = plugin_trim,
   .zero = plugin_zero,
+  .extents = plugin_extents,
 };
 
 /* Register and load a plugin. */
