@@ -13,7 +13,7 @@
 "       [--tls-psk /path/to/pskfile] [--tls-verify-peer]\n"
 "       [-U|--unix SOCKET] [-u|--user USER]\n"
 "       [-v|--verbose] [-V|--version]\n"
-"       PLUGIN [KEY=VALUE [KEY=VALUE [...]]]\n"
+"       PLUGIN [[KEY=]VALUE [KEY=VALUE [...]]]\n"
 "\n"
 "nbdkit --dump-config\n"
 "\n"
