@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -38,18 +37,15 @@ source ./functions.sh
 set -e
 set -x
 
-files="data-7E.out data-7E.pid data-7E.sock"
+requires qemu-io --version
+
+sock=`mktemp -u`
+files="data-7E.out data-7E.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
 
-# Test that qemu-io works
-if ! qemu-io --help >/dev/null; then
-    echo "$0: missing or broken qemu-io"
-    exit 77
-fi
-
 # Run nbdkit.
-start_nbdkit -P data-7E.pid -U data-7E.sock --export= \
+start_nbdkit -P data-7E.pid -U $sock --export= \
        --filter=partition \
        data size=7E partition=1 \
        data="
@@ -90,7 +86,7 @@ start_nbdkit -P data-7E.pid -U data-7E.sock --export= \
    "
 
 # Since we're reading the empty first partition, any read returns zeroes.
-qemu-io -r -f raw 'nbd+unix://?socket=data-7E.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 498 16' | grep -E '^[[:xdigit:]]+:' > data-7E.out
 if [ "$(cat data-7E.out)" != "000001f2:  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................" ]
 then
