@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,23 +36,20 @@ source ./functions.sh
 set -e
 set -x
 
-files="data-file.out data-file.pid data-file.sock data-hello.txt"
+requires qemu-io --version
+
+sock=`mktemp -u`
+files="data-file.out data-file.pid $sock data-hello.txt"
 rm -f $files
 cleanup_fn rm -f $files
 
-# Test that qemu-io works
-if ! qemu-io --help >/dev/null; then
-    echo "$0: missing or broken qemu-io"
-    exit 77
-fi
-
 rm -f data-hello.txt
 for i in {0..1000}; do
     echo -n "hello " >> data-hello.txt
 done
 
 # Run nbdkit.
-start_nbdkit -P data-file.pid -U data-file.sock \
+start_nbdkit -P data-file.pid -U $sock \
        --filter=partition \
        data partition=1 \
        size=1M \
@@ -64,7 +60,7 @@ start_nbdkit -P data-file.pid -U data-file.sock \
    "
 
 # Since we're reading the empty first partition, any read returns zeroes.
-qemu-io -r -f raw 'nbd+unix://?socket=data-file.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 0 900' | grep -E '^[[:xdigit:]]+:' > data-file.out
 if [ "$(cat data-file.out)" != "00000000:  68 65 6c 6c 6f 20 68 65 6c 6c 6f 20 68 65 6c 6c  hello.hello.hell
 00000010:  6f 20 68 65 6c 6c 6f 20 68 65 6c 6c 6f 20 68 65  o.hello.hello.he
