@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,21 +36,18 @@ source ./functions.sh
 set -e
 set -x
 
-files="data-raw.out data-raw.pid data-raw.sock"
+requires qemu-io --version
+
+sock=`mktemp -u`
+files="data-raw.out data-raw.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
 
-# Test that qemu-io works
-if ! qemu-io --help >/dev/null; then
-    echo "$0: missing or broken qemu-io"
-    exit 77
-fi
-
 # Run nbdkit.
-start_nbdkit -P data-raw.pid -U data-raw.sock --export "" \
+start_nbdkit -P data-raw.pid -U $sock --export "" \
        data raw=123 size=512
 
-qemu-io -r -f raw 'nbd+unix://?socket=data-raw.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 0 512' | grep -E '^[[:xdigit:]]+:' > data-raw.out
 if [ "$(cat data-raw.out)" != "00000000:  31 32 33 00 00 00 00 00 00 00 00 00 00 00 00 00  123.............
 00000010:  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................
