@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2016-2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,32 +36,15 @@
 source ./functions.sh
 set -e
 
+requires ip -V
+requires qemu-img --version
+requires qemu-img info --image-opts driver=file,filename=functions.sh
+
 rm -f ip.pid ipv4.out ipv6.out
 cleanup_fn rm -f ip.pid ipv4.out ipv6.out
 
-# Don't fail if certain commands aren't available.
-if ! ss --version; then
-    echo "$0: 'ss' command not available"
-    exit 77
-fi
-if ! command -v qemu-img; then
-    echo "$0: 'qemu-img' command not available"
-    exit 77
-fi
-if ! qemu-img --help | grep -- --image-opts; then
-    echo "$0: 'qemu-img' command does not have the --image-opts option"
-    exit 77
-fi
-if ! ip -V; then
-    echo "$0: 'ip' command not available"
-    exit 77
-fi
-
 # Find an unused port to listen on.
-for port in {49152..65535}; do
-    if ! ss -ltn | grep -sqE ":$port\b"; then break; fi
-done
-echo picked unused port $port
+pick_unused_port
 
 # By default nbdkit will listen on all available interfaces, ie.
 # IPv4 and IPv6.
