@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -61,7 +60,7 @@
 
 #include "byte-swapping.h"
 #include "exit-with-parent.h"
-#include "protocol.h"           /* From nbdkit core. */
+#include "protocol.h"
 
 /* Declare program_name. */
 #if HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME == 1
@@ -92,7 +91,7 @@ main (int argc, char *argv[])
   struct new_handshake_finish handshake_finish;
   uint16_t eflags;
   struct request request;
-  struct reply reply;
+  struct simple_reply reply;
   char data[512];
 
 #ifndef HAVE_EXIT_WITH_PARENT
@@ -355,6 +354,12 @@ main (int argc, char *argv[])
      "filter1: test_layers_filter_can_multi_conn",
      "test_layers_plugin_can_multi_conn",
      NULL);
+  log_verify_seen_in_order
+    ("filter3: test_layers_filter_can_extents",
+     "filter2: test_layers_filter_can_extents",
+     "filter1: test_layers_filter_can_extents",
+     "test_layers_plugin_can_extents",
+     NULL);
 
   fprintf (stderr, "%s: protocol connected\n", program_name);
 
@@ -520,6 +525,12 @@ main (int argc, char *argv[])
      "test_layers_plugin_zero",
      NULL);
 
+  /* XXX We should test NBD_CMD_BLOCK_STATUS here.  However it
+   * requires that we negotiate structured replies and base:allocation
+   * in the handshake, and the format of the reply is more complex
+   * than what we expect in this naive test.
+   */
+
   /* Close the connection. */
   fprintf (stderr, "%s: closing the connection\n", program_name);
   request.type = htobe16 (NBD_CMD_DISC);
