@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -34,58 +33,28 @@
 # Test the memory plugin with the largest possible size supported
 # by qemu and nbdkit.
 
+source ./functions.sh
 set -e
 
-files="memory-largest-for-qemu.out memory-largest-for-qemu.pid memory-largest-for-qemu.sock"
-rm -f $files
+requires qemu-io --version
 
-# Test that qemu-io works
-if ! qemu-io --help >/dev/null; then
-    echo "$0: missing or broken qemu-io"
-    exit 77
-fi
+sock=`mktemp -u`
+files="memory-largest-for-qemu.out memory-largest-for-qemu.pid $sock"
+rm -f $files
+cleanup_fn rm -f $files
 
 # Run nbdkit with memory plugin.
 # size = (2^63-1) & ~511 which is the largest supported by qemu.
-nbdkit -f -v -D memory.dir=1 \
-       -P memory-largest-for-qemu.pid -U memory-largest-for-qemu.sock \
-       memory size=9223372036854775296 &
-
-# We may have to wait a short time for the pid file to appear.
-for i in `seq 1 10`; do
-    if test -f memory-largest-for-qemu.pid; then
-        break
-    fi
-    sleep 1
-done
-if ! test -f memory-largest-for-qemu.pid; then
-    echo "$0: PID file was not created"
-    exit 1
-fi
-
-pid="$(cat memory-largest-for-qemu.pid)"
-
-# Kill the nbdkit process on exit.
-cleanup ()
-{
-    status=$?
-    trap '' INT QUIT TERM EXIT ERR
-    echo $0: cleanup: exit code $status
-
-    kill $pid
-    rm -f $files
-
-    exit $status
-}
-trap cleanup INT QUIT TERM EXIT ERR
+start_nbdkit -P memory-largest-for-qemu.pid -U $sock \
+       memory size=9223372036854775296
 
 # Write some stuff to the beginning, middle and end.
-qemu-io -f raw 'nbd+unix://?socket=memory-largest-for-qemu.sock' \
+qemu-io -f raw "nbd+unix://?socket=$sock" \
         -c 'w -P 1 0 512' \
         -c 'w -P 2 1000000001 65536' \
         -c 'w -P 3 9223372036854774784 512'
 
-qemu-io -r -f raw 'nbd+unix://?socket=memory-largest-for-qemu.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 0 512' | grep -E '^[[:xdigit:]]+:' > memory-largest-for-qemu.out
 if [ "$(cat memory-largest-for-qemu.out)" != "00000000:  01 01 01 01 01 01 01 01 01 01 01 01 01 01 01 01  ................
 00000010:  01 01 01 01 01 01 01 01 01 01 01 01 01 01 01 01  ................
@@ -125,7 +94,7 @@ then
     exit 1
 fi
 
-qemu-io -r -f raw 'nbd+unix://?socket=memory-largest-for-qemu.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 1000000001 512' | grep -E '^[[:xdigit:]]+:' > memory-largest-for-qemu.out
 if [ "$(cat memory-largest-for-qemu.out)" != "3b9aca01:  02 02 02 02 02 02 02 02 02 02 02 02 02 02 02 02  ................
 3b9aca11:  02 02 02 02 02 02 02 02 02 02 02 02 02 02 02 02  ................
@@ -166,7 +135,7 @@ then
 fi
 
 # This block of memory was not set, so it should read back as zeroes.
-qemu-io -r -f raw 'nbd+unix://?socket=memory-largest-for-qemu.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 2000000000 512' | grep -E '^[[:xdigit:]]+:' > memory-largest-for-qemu.out
 if [ "$(cat memory-largest-for-qemu.out)" != "77359400:  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................
 77359410:  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................
@@ -206,7 +175,7 @@ then
     exit 1
 fi
 
-qemu-io -r -f raw 'nbd+unix://?socket=memory-largest-for-qemu.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 9223372036854774784 512' | grep -E '^[[:xdigit:]]+:' > memory-largest-for-qemu.out
 if [ "$(cat memory-largest-for-qemu.out)" != "7ffffffffffffc00:  03 03 03 03 03 03 03 03 03 03 03 03 03 03 03 03  ................
 7ffffffffffffc10:  03 03 03 03 03 03 03 03 03 03 03 03 03 03 03 03  ................
@@ -245,5 +214,3 @@ then
     cat memory-largest-for-qemu.out
     exit 1
 fi
-
-# The cleanup() function is called implicitly on exit.
