@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,24 +36,21 @@
 source ./functions.sh
 set -e
 
-files="memory-largest.out memory-largest.pid memory-largest.sock"
+requires qemu-io --version
+
+sock=`mktemp -u`
+files="memory-largest.out memory-largest.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
 
-# Test that qemu-io works
-if ! qemu-io --help >/dev/null; then
-    echo "$0: missing or broken qemu-io"
-    exit 77
-fi
-
 # Run nbdkit with memory plugin.
 # size = 2^63-1
-start_nbdkit -P memory-largest.pid -U memory-largest.sock \
+start_nbdkit -P memory-largest.pid -U $sock \
        memory size=9223372036854775807
 
 # qemu cannot open this image!
 #
-#   can't open device nbd+unix://?socket=memory-largest.sock: Could not get image size: File too large
+#   can't open device nbd+unix://?socket=$sock: Could not get image size: File too large
 #
 # Therefore we skip the remainder of this test (in effect, testing
 # only that nbdkit can create the file).
