@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2017-2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -34,14 +33,8 @@
 source ./functions.sh
 
 # Check file-data was created by Makefile and qemu-io exists.
-if ! test -f file-data; then
-    echo "$0: missing file-data"
-    exit 77
-fi
-if ! qemu-io --version; then
-    echo "$0: missing qemu-io"
-    exit 77
-fi
+requires test -f file-data
+requires qemu-io --version
 
 cleanup_fn rm -f test-parallel-file.data test-parallel-file.out
 
