@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -33,7 +32,7 @@
 
 # Test the partitioning plugin.
 #
-# Test 4: Test > 128 partitions.
+# Test 4: Test > 128 partitions using GPT.
 #
 # virtio-scsi (used by libguestfs) doesn't support more than 15
 # partitions.  In fact the only client which supports this is our own
@@ -43,11 +42,7 @@ source ./functions.sh
 set -e
 set -x
 
-# Test that qemu-img works
-if ! qemu-img --help >/dev/null; then
-    echo "$0: missing or broken qemu-img"
-    exit 77
-fi
+requires qemu-img --version
 
 # This test requires the partitioning plugin to open at least 768
 # files (say 800 to make it a round number).  On OpenBSD the limit on
