@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2017 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -35,15 +34,8 @@ source ./functions.sh
 set -e
 set -x
 
-# Don't fail if certain commands aren't available.
-if ! ss --version; then
-    echo "$0: 'ss' command not available"
-    exit 77
-fi
-if ! command -v qemu-img > /dev/null; then
-    echo "$0: 'qemu-img' command not available"
-    exit 77
-fi
+requires qemu-img --version
+
 if ! qemu-img --help | grep -- --object; then
     echo "$0: 'qemu-img' command does not have the --object option"
     exit 77
@@ -66,10 +58,7 @@ fi
 # Unfortunately qemu cannot do TLS over a Unix domain socket (nbdkit
 # probably can, although it is not tested).  Find an unused port to
 # listen on.
-for port in {50000..65535}; do
-    if ! ss -ltn | grep -sqE ":$port\b"; then break; fi
-done
-echo picked unused port $port
+pick_unused_port
 
 cleanup_fn rm -f tls.pid tls.out
 start_nbdkit -P tls.pid -p $port -n --tls=require \
