@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
 # Copyright (C) 2018 Red Hat Inc.
-# All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,23 +36,20 @@ source ./functions.sh
 set -e
 set -x
 
-files="truncate2.out truncate2.pid truncate2.sock"
+requires qemu-io --version
+
+sock=`mktemp -u`
+files="truncate2.out truncate2.pid $sock"
 rm -f $files
 cleanup_fn rm -f $files
 
-# Test that qemu-io works
-if ! qemu-io --help >/dev/null; then
-    echo "$0: missing or broken qemu-io"
-    exit 77
-fi
-
 # Run nbdkit with pattern plugin and truncate filter in front.
-start_nbdkit -P truncate2.pid -U truncate2.sock \
+start_nbdkit -P truncate2.pid -U $sock \
        --filter=truncate \
        pattern size=503 \
        round-up=512
 
-qemu-io -r -f raw 'nbd+unix://?socket=truncate2.sock' \
+qemu-io -r -f raw "nbd+unix://?socket=$sock" \
         -c 'r -v 0 512' | grep -E '^[[:xdigit:]]+:' > truncate2.out
 if [ "$(cat truncate2.out)" != "00000000:  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 08  ................
 00000010:  00 00 00 00 00 00 00 10 00 00 00 00 00 00 00 18  ................
