@@ -1,6 +1,5 @@
 /* nbdkit
  * Copyright (C) 2017-2018 Red Hat Inc.
- * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -32,20 +31,24 @@
  */
 
 /*------------------------------------------------------------
- * This wrapper lets you run nbdkit from the source directory.
+ * This is not nbdkit.  This is a wrapper which lets you run nbdkit
+ * from the source directory without installing nbdkit.
  *
  * You can use either:
- * ./nbdkit file [arg=value] [arg=value] ...
+ *
+ *   ./nbdkit file [arg=value] [arg=value] ...
+ *
  * or:
+ *
  *   /path/to/nbdkit file [arg=value] [arg=value] ...
  *
- * Or you can set $PATH to include the nbdkit source directory and run
+ * or you can set $PATH to include the nbdkit source directory and run
  * the bare "nbdkit" command without supplying the full path.
  *
  * The wrapper modifies the bare plugin name (eg. "file") to be the
  * full path to the locally compiled plugin.  If you don't use this
  * program and run server/nbdkit directly then it will pick up the
- * installed plugins which is not usually what you want.
+ * globally installed plugins which is usually not what you want.
  *
  * This program is also used to run the tests (make check).
  *
@@ -66,6 +69,7 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <limits.h>
+#include <time.h>
 
 #include "options.h"
 
@@ -73,6 +77,16 @@
 static const char **cmd;
 static size_t len;
 
+/* Plugins written in scripting languages (only Perl right now) need
+ * to be rewritten on the command line in a different way from plugins
+ * written in C.  So we have to list those here.
+ */
+static bool
+is_perl_plugin (const char *name)
+{
+  return strcmp (name, "example4") == 0 || strcmp (name, "tar") == 0;
+}
+
 static void
 passthru (const char *s)
 {
@@ -119,6 +133,9 @@ main (int argc, char *argv[])
 {
   bool verbose = false;
   char *s;
+  time_t t;
+  unsigned tu;
+  char ts[32];
 
   /* If NBDKIT_VALGRIND=1 is set in the environment, then we run the
    * program under valgrind.  This is used by the tests.  Similarly if
@@ -212,8 +229,7 @@ main (int argc, char *argv[])
      */
     if (is_short_name (argv[optind])) {
       /* Special plugins written in Perl. */
-      if (strcmp (argv[optind], "example4") == 0 ||
-          strcmp (argv[optind], "tar") == 0) {
+      if (is_perl_plugin (argv[optind])) {
         passthru_format ("%s/plugins/perl/.libs/nbdkit-perl-plugin.so",
                          builddir);
         passthru_format ("%s/plugins/%s/nbdkit-%s-plugin",
@@ -237,6 +253,19 @@ main (int argc, char *argv[])
   if (verbose)
     print_command ();
 
+  /* This is a cheap way to find some use-after-free and uninitialized
+   * read problems when using glibc, and doesn't affect normal
+   * operation or other libc.  We don't overwrite existing values so
+   * this can be disabled or overridden at runtime.
+   */
+  setenv ("MALLOC_CHECK_", "1", 0);
+  time (&t);
+  tu = t;
+  tu %= 255;
+  tu++;
+  snprintf (ts, sizeof ts, "%u", tu);
+  setenv ("MALLOC_PERTURB_", ts, 0);
+
   /* Run the final command. */
   execvp (cmd[0], (char **) cmd);
   perror (cmd[0]);
